//******************************************
//  decompressor.c -- Data decompression
//  program.
//  Modified by Phillip Weissfloch,
//  Sensoray Co., Inc., 2002
//
//  Based on original:
//  SIXPACK.C -- Data compression program
//  Written by Philip G. Gage, April 1991
//
//******************************************

#include "decomp_six.h"

#define MAXFREQ    2000   // Max frequency count before table reset
#define MINCOPY       3   // Shortest string copy length
#define MAXCOPY      64   // Longest string copy length
#define COPYRANGES    6   // Number of string copy distance bit ranges
static short copybits[COPYRANGES] =
{
    4, 6, 8, 10, 12, 14
};   // Distance bits

#define CODESPERRANGE (MAXCOPY - MINCOPY + 1)

#define NIL -1                    // End of linked list marker

// Adaptive Huffman variables
#define TERMINATE 256             // EOF code
#define FIRSTCODE 257             // First code for copy lengths
#define MAXCHAR ( FIRSTCODE + COPYRANGES * CODESPERRANGE - 1)
#define SUCCMAX (MAXCHAR + 1)
#define TWICEMAX (2 * MAXCHAR + 1)
#define ROOT 1

typedef struct
{
    int           copymin[COPYRANGES];
    int           copymax[COPYRANGES];
    int           maxdistance;
    int           maxsize;
    unsigned char *buffer;                   // Text buffer
    short         left [MAXCHAR  + 1];       // Huffman tree
    short         right[MAXCHAR  + 1];
    short         up   [TWICEMAX + 1];
    short         freq [TWICEMAX + 1];

    int           input_bit_count;           // Input bits buffered
    int           input_bit_buffer;          // Input buffer
    long          bytes_in;                  // File size counters
    long          bytes_out;
    
    unsigned char *indata;                   // Pointer to compressed data input buffer
    int           in_index;                  // Input buffer index

	// Used only by read_decompressor function
	unsigned char read_leftover_buffer[256]; // buffer to temporarily hold excess data
    int           read_leftover;             // count of excess data bytes
	int           read_n;                    // ???
	int           read_done;                 // Non-zero when all data has been read
} DECDATA;

//static DECDATA dd;

// Read a bit from input file 
static int input_bit( DECDATA *decdata )
{
    int bit;

    if ( decdata->input_bit_count-- == 0 )
    {
        // read the compressed data buffer
        decdata->input_bit_buffer = decdata->indata[decdata->in_index++];

        ++decdata->bytes_in;
        decdata->input_bit_count = 7;
    }
    bit = ( decdata->input_bit_buffer & 0x80 ) != 0;
    decdata->input_bit_buffer <<= 1;
    return bit;
}

// Read multibit code from input file
static int input_code( DECDATA *decdata, int bits )
{
    int i;
    int bit = 1;
    int code = 0;

    for ( i = 0;  i < bits; i++)
    {
        if ( input_bit( decdata ) )
            code |= bit;
        bit <<= 1;
    }
    return code;
}

// Initialize data for compression or decompression
static void initialize( DECDATA *decdata )
{
    int i;
    int j;
	unsigned char *pdec = (unsigned char *) decdata;

    // Initialize the data structure
	for ( i = 0; i < sizeof( DECDATA ); i++ )
    	pdec[i] = 0;

    // Initialize Huffman frequency tree
    for ( i = 2; i <= TWICEMAX; i++ )
    {
        decdata->up[i]   = i / 2;
        decdata->freq[i] = 1;
    }
    for ( i = 1; i <= MAXCHAR; i++ )
    {
        decdata->left[i]  = 2 * i;
        decdata->right[i] = 2 * i + 1;
    }

    // Initialize copy distance ranges
    j = 0;
    for ( i = 0; i < COPYRANGES; i++ )
    {
        decdata->copymin[i] = j;
        j                  += 1 << copybits[i];
        decdata->copymax[i] = j - 1;
    }
    decdata->maxdistance = j - 1;
    decdata->maxsize     = decdata->maxdistance + MAXCOPY;
}

// Update frequency counts from leaf to root
static void update_freq( DECDATA *decdata, int a, int b )
{
    do
    {
        decdata->freq[decdata->up[a]] = decdata->freq[a] + decdata->freq[b];
        a = decdata->up[a];
        if ( a != ROOT )
        {
            if ( decdata->left[decdata->up[a]] == a )
                b = decdata->right[decdata->up[a]];
            else
                b = decdata->left[decdata->up[a]];
        }
    }
    while ( a != ROOT );

    // Periodically scale frequencies down by half to avoid overflow
    // This also provides some local adaption and better compression
    if ( decdata->freq[ROOT] == MAXFREQ )
        for ( a = 1; a <= TWICEMAX; a++ )
            decdata->freq[a] >>= 1;
}

// Update Huffman model for each character code
static void update_model( DECDATA *decdata, int code )
{
    int a;
    int b;
    int c;
    int ua;
    int uua;

    a = code + SUCCMAX;
    ++decdata->freq[a];
    if ( decdata->up[a] != ROOT )
    {
        ua = decdata->up[a];
        if ( decdata->left[ua] == a )
            update_freq( decdata, a, decdata->right[ua] );
        else
            update_freq( decdata, a, decdata->left[ua] );
        do
        {
            uua = decdata->up[ua];
            if ( decdata->left[uua] == ua )
                b = decdata->right[uua];
            else
                b = decdata->left[uua];

            // If high freq lower in tree, swap nodes
            if ( decdata->freq[a] > decdata->freq[b] )
            {
                if ( decdata->left[uua] == ua )
                    decdata->right[uua] = (short)a;
                else
                    decdata->left[uua] = (short)a;
                if ( decdata->left[ua] == a )
                {
                    decdata->left[ua] = (short)b;
                    c = decdata->right[ua];
                }
                else
                {
                    decdata->right[ua] = (short)b;
                    c = decdata->left[ua];
                }
                decdata->up[b] = (short)ua;
                decdata->up[a] = (short)uua;
                update_freq( decdata, b, c );
                a = b;
            }
            a  = decdata->up[a];
            ua = decdata->up[a];
        }
        while ( ua != ROOT );
    }
}

// Uncompress a character code from input stream
static int uncompress( DECDATA *decdata )
{
    int a   = ROOT;

    do
    {
        if ( input_bit( decdata ) )
            a = decdata->right[a];
        else
            a = decdata->left[a];
    }
    while ( a <= MAXCHAR );

    update_model( decdata, a - SUCCMAX );

    return a - SUCCMAX;
}

// Open the decompressor. 'indata' points to an
// array of compressed data bytes. 'your_malloc' is
// a pointer to the 'malloc' function on your system.
// A 'file' handle to the decompressor
// is returned. Zero is returned on error.
void* open_decompressor( unsigned char *indata, void *your_malloc )
{
	DECDATA *decdata;
	void * (*dc_malloc)(int);

	dc_malloc = your_malloc;

	// allocate DECDATA structure
	decdata = (DECDATA *) dc_malloc( sizeof(DECDATA) );
	if ( decdata <= 0 )
		return 0;

	// initialize DECDATA structure
    initialize( decdata );

	// set input buffer pointer
    decdata->indata = indata;

	// allocate work buffer
    decdata->buffer = (unsigned char *) dc_malloc( decdata->maxsize );
    if (decdata->buffer <= 0)
	    return 0;
    
    decdata->read_done = 0;

    return decdata;
}

// Close the decompressor. 'your_free' is a pointer to the 'free' function on
// your system.
void close_decompressor( void *handle, void *your_free )
{
	DECDATA *decdata = (DECDATA *) handle;
	void (*dc_free)(void *);

	dc_free = your_free;

	// Release the allocated buffers
    dc_free( decdata->buffer );
    dc_free( decdata );
}


// Read as many bytes (count) of uncompressed data (buffer) as desired.
// Returns the number of bytes actually read. If the number is less than
// the number asked for, the end has been reached. A return of -1 indicates
// that there is no data to be read.
int read_decompressor( void *handle, void *buffer, int count )
{
	DECDATA *decdata = (DECDATA *) handle;
    int c;
    int buffer_idx = 0;
    int dist;
    int index;
	int i;
	int j;
	int k;
	int len;
	unsigned char *pbuffer = (unsigned char *) buffer;

	// Return -1 if there is no data to be read
	if ( decdata->read_done )
		return -1;

	// If there is any data in the leftover buffer, place it in
	// the output buffer before getting new data.
	if ( decdata->read_leftover > 0 )
	{
		if ( decdata->read_leftover >= count )
		{
			for ( i = 0; i < count; i++ )
			{
				pbuffer[i] = decdata->read_leftover_buffer[i];
			}

			if ( decdata->read_leftover > count )
			{
				// move the leftover data up to the beginning of the leftover buffer
				for ( i = 0; i < decdata->read_leftover - count; i++ )
				{
					decdata->read_leftover_buffer[i] = decdata->read_leftover_buffer[count + i];
				}
			}
			decdata->read_leftover -= count;

			return count;
		}
		else
		{
			for ( i = 0; i < decdata->read_leftover; i++ )
			{
				pbuffer[i] = decdata->read_leftover_buffer[i];
			}
			buffer_idx = decdata->read_leftover;

			decdata->read_leftover = 0;
		}
	}

    while ( ( c = uncompress( decdata ) ) != TERMINATE )
    {
        if ( c < 256 )
        {
            // Single literal character ?
            pbuffer[buffer_idx++] = c & 0xff;
            ++decdata->bytes_out;
            decdata->buffer[decdata->read_n++] = (unsigned char)c;

            if ( decdata->read_n == decdata->maxsize )
                decdata->read_n = 0;

			if ( buffer_idx >= count )
			{
				return buffer_idx;
			}
        }
        else
        {
            // Else string copy length/distance codes
            index = ( c - FIRSTCODE ) / CODESPERRANGE;
            len   = c - FIRSTCODE + MINCOPY - index * CODESPERRANGE;
            dist  = input_code( decdata, copybits[index] ) + len + decdata->copymin[index];
            j     = decdata->read_n;
            k     = decdata->read_n - dist;

            if ( k < 0 )
                k += decdata->maxsize;

            for ( i = 0; i < len; i++ )
            {
				if ( buffer_idx < count )
            		pbuffer[buffer_idx++] = decdata->buffer[k];
				else
					decdata->read_leftover_buffer[decdata->read_leftover++] = decdata->buffer[k];

                ++decdata->bytes_out;
                decdata->buffer[j++] = decdata->buffer[k++];

                if ( j == decdata->maxsize )
                    j = 0;
                if ( k == decdata->maxsize )
                    k = 0;

            }
            decdata->read_n += len;
            if ( decdata->read_n >= decdata->maxsize )
                decdata->read_n -= decdata->maxsize;

			if ( buffer_idx == count )
			{
				return buffer_idx;
			}
        }
    }
	decdata->read_done = 1;

	return buffer_idx;
}


