#ifndef __VIDCAP_H
#define __VIDCAP_H
/* video capture parameters */
/* based on Linux videodev2.h VIDCAP interface */

/*  Four-character-code (FOURCC) */
#define vidcap_fourcc(a,b,c,d)\
        (((unsigned long)(a)<<0)|((unsigned long)(b)<<8)|((unsigned long)(c)<<16)|((unsigned long)(d)<<24))


/*
 *	E N U M S
 */

enum vidcap_field {
	VIDCAP_FIELD_ANY        = 0, /* driver can choose from none,
				      top, bottom, interlaced
				      depending on whatever it thinks
				      is approximate ... */
	VIDCAP_FIELD_NONE       = 1, /* this device has no fields ... */
	VIDCAP_FIELD_TOP        = 2, /* top field only */
	VIDCAP_FIELD_BOTTOM     = 3, /* bottom field only */
	VIDCAP_FIELD_INTERLACED = 4, /* both fields interlaced */
	VIDCAP_FIELD_SEQ_TB     = 5, /* both fields sequential into one
				      buffer, top-bottom order */
	VIDCAP_FIELD_SEQ_BT     = 6, /* same as above + bottom-top order */
	VIDCAP_FIELD_ALTERNATE  = 7, /* both fields alternating into
				      separate buffers */
};


enum vidcap_buf_type {
	VIDCAP_BUF_TYPE_VIDEO_CAPTURE  = 1,
	VIDCAP_BUF_TYPE_VIDEO_OUTPUT   = 2,
	VIDCAP_BUF_TYPE_VIDEO_OVERLAY  = 3,
	VIDCAP_BUF_TYPE_VBI_CAPTURE    = 4,
	VIDCAP_BUF_TYPE_VBI_OUTPUT     = 5,
	VIDCAP_BUF_TYPE_PRIVATE        = 0x80,
};



#define VIDCAP_FIELD_HAS_TOP(field)	\
	((field) == VIDCAP_FIELD_TOP 	||\
	 (field) == VIDCAP_FIELD_INTERLACED ||\
	 (field) == VIDCAP_FIELD_SEQ_TB	||\
	 (field) == VIDCAP_FIELD_SEQ_BT)
#define VIDCAP_FIELD_HAS_BOTTOM(field)	\
	((field) == VIDCAP_FIELD_BOTTOM 	||\
	 (field) == VIDCAP_FIELD_INTERLACED ||\
	 (field) == VIDCAP_FIELD_SEQ_TB	||\
	 (field) == VIDCAP_FIELD_SEQ_BT)
#define VIDCAP_FIELD_HAS_BOTH(field)	\
	((field) == VIDCAP_FIELD_INTERLACED ||\
	 (field) == VIDCAP_FIELD_SEQ_TB	||\
	 (field) == VIDCAP_FIELD_SEQ_BT)


/* see also http://vektor.theorem.ca/graphics/ycbcr/ */
enum vidcap_colorspace {
	/* ITU-R 601 -- broadcast NTSC/PAL */
	VIDCAP_COLORSPACE_SMPTE170M     = 1,

	/* 1125-Line (US) HDTV */
	VIDCAP_COLORSPACE_SMPTE240M     = 2,

	/* HD and modern captures. */
	VIDCAP_COLORSPACE_REC709        = 3,
	
	/* broken BT878 extents (601, luma range 16-253 instead of 16-235) */
	VIDCAP_COLORSPACE_BT878         = 4,
	
	/* These should be useful.  Assume 601 extents. */
	VIDCAP_COLORSPACE_470_SYSTEM_M  = 5,
	VIDCAP_COLORSPACE_470_SYSTEM_BG = 6,
	
	/* I know there will be cameras that send this.  So, this is
	 * unspecified chromaticities and full 0-255 on each of the
	 * Y'CbCr components
	 */
	VIDCAP_COLORSPACE_JPEG          = 7,
	
	/* For RGB colourspaces, this is probably a good start. */
	VIDCAP_COLORSPACE_SRGB          = 8,
};


/*
 *	V I D E O   I M A G E   F O R M A T
 */

#define MAXCLIPS    16

struct vidcap_rect
{
	int   left;
	int   top;
	int   width;
	int   height;
};


struct vidcap_clipinfo  
{
    int clipcount;
    struct vidcap_rect clips[MAXCLIPS];
};


struct vidcap_pix_format
{
	unsigned long         	type;
	unsigned long         	capability;
	unsigned long         	flags;
	unsigned long         	base;
	int fourcc;
	unsigned long         	left;
	unsigned long         	top;
	unsigned long         	width;
	unsigned long	        height;
	unsigned long	        pixelformat;
	enum vidcap_field  	    field;
	unsigned long           bytesperline;	/* for padding, zero if unused */
	unsigned long          	sizeimage;
    enum vidcap_colorspace	colorspace;
};

// common formats
/*           Pixel format    FOURCC                  depth  Description   */
#define VIDCAP_PIX_FMT_RGB332  vidcap_fourcc('R','G','B','1') /*  8  RGB-3-3-2     */
#define VIDCAP_PIX_FMT_RGB555  vidcap_fourcc('R','G','B','O') /* 16  RGB-5-5-5     */
#define VIDCAP_PIX_FMT_RGB565  vidcap_fourcc('R','G','B','P') /* 16  RGB-5-6-5     */
#define VIDCAP_PIX_FMT_RGB555X vidcap_fourcc('R','G','B','Q') /* 16  RGB-5-5-5 BE  */
#define VIDCAP_PIX_FMT_RGB565X vidcap_fourcc('R','G','B','R') /* 16  RGB-5-6-5 BE  */
#define VIDCAP_PIX_FMT_BGR24   vidcap_fourcc('B','G','R','3') /* 24  BGR-8-8-8     */
#define VIDCAP_PIX_FMT_RGB24   vidcap_fourcc('R','G','B','3') /* 24  RGB-8-8-8     */
#define VIDCAP_PIX_FMT_BGR32   vidcap_fourcc('B','G','R','4') /* 32  BGR-8-8-8-8   */
#define VIDCAP_PIX_FMT_RGB32   vidcap_fourcc('R','G','B','4') /* 32  RGB-8-8-8-8   */
#define VIDCAP_PIX_FMT_GREY    vidcap_fourcc('G','R','E','Y') /*  8  Greyscale     */
#define VIDCAP_PIX_FMT_YVU410  vidcap_fourcc('Y','V','U','9') /*  9  YVU 4:1:0     */
#define VIDCAP_PIX_FMT_YVU420  vidcap_fourcc('Y','V','1','2') /* 12  YVU 4:2:0     */
#define VIDCAP_PIX_FMT_YUYV    vidcap_fourcc('Y','U','Y','V') /* 16  YUV 4:2:2     */
#define VIDCAP_PIX_FMT_UYVY    vidcap_fourcc('U','Y','V','Y') /* 16  YUV 4:2:2     */
#define VIDCAP_PIX_FMT_YUV422P vidcap_fourcc('4','2','2','P') /* 16  YVU422 planar */
#define VIDCAP_PIX_FMT_YUV411P vidcap_fourcc('4','1','1','P') /* 16  YVU411 planar */
#define VIDCAP_PIX_FMT_Y41P    vidcap_fourcc('Y','4','1','P') /* 12  YUV 4:1:1     */


#endif 
